# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '..'}

packages = \
['controller',
 'controller.core',
 'controller.methods',
 'controller.models',
 'controller.plugin']

package_data = \
{'': ['*']}

modules = \
['__init__']
install_requires = \
['SQLAlchemy>=1.4.29,<2.0.0',
 'aiosqlite>=0.17.0,<0.18.0',
 'anyutils>=0.2.3,<0.3.0',
 'nonebot2>=2.0.0-beta.1,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-pctrl',
    'version': '0.2.3',
    'description': '',
    'long_description': '# Plugins controller for nonebot2\n## 简介\n\n基于 Hook 的插件管理器 , 提供常用的管理功能\n\n使用异步 SQLAlchemy 方言实现持久化\n\n功能持续增加中\n\n## 计划\n\n- [X] ~~全局插件开关~~ 可以通过设置`ban`来实现\n- [X] 设置`ban`人/群或人群\n- [X] 设置冷却时间(通过配置文件)\n- [X] 适配b1\n- [X] 模块化\n- [X] 使用 `core` + `pydantic` 代替 `ORM`\n- [ ] 冷却时间信息显示与命令修改\n\n> 欢迎 issue 提出更多需求\n\n## 使用说明\n\n1. 本插件初始化于 `websocket` 连接时 , http协议上报未经测试\n2. 导入时**注意大小写** `nonebot_plugin_PCtrl`\n3. 所有命令只能由 `SUPERUSER` 激活\n4. 数据库初始化未完成之前所有事件(除元事件)都会被忽略\n\n## 功能\n### ban\n\n- `/listban` 列出存在的 ban 的信息\n- `/ban`\n- `/unban`\n  \n以上命令皆有三个参数 \n\n1. `-u` : QQ号\n2. `-g` : 群号\n3. `-p` : 插件名称\n\n如果未输入参数则认为作用在全局 , 例如 `\\ban -u 123456` 的意义为禁用 QQ 为 123456 的人在所有群使用所有插件的权限\n\n### cool\n\n本部分提供三个级别的控制 , plugin , matcher , function\n\n对于插件级别的控制 , 通过配置文件实现 , 详见下\n\n对于 matcher 级别的控制 , 通过跨插件方法导入类 `cooling`\n```py\nfrom nonebot.plugin import require\nreq = require("nonebot_plugin_PCtrl")\ncmd = req.cooling.cool_matcher(5, on_keyword({"jls", "jiangly"}, priority=10)) # 对一个 matcher 启用冷却\n@cmd.handle()\nasync def _(bot, event, state): pass\n```\n\n对于 function 级别的控制 , 通过跨插件方法导入类 `cooling`\n```py\nfrom nonebot.plugin import require\nreq = require("admin.nonebot_plugin_PCtrl")\ncoolen_async = req.cooling.cool_async\n\n@coolen_async(5)\nasync def _(bot, event, state): pass\n```\n\n对于通过 `cooling` 类实现的冷却 , 当未到冷却时间调用时会 `raise CoolingError`\n```py\n# 参见\nfrom anyutils import CoolingError\n```\n### sender (工具类)\n\n通过跨插件方法导入类 `sender` , 源码位于 `plugin/sender.py`\n\n```py\n# 当有没有处理的 `Execption` 时汇报到群聊(见配置)\n@sender.when_raise(log)\nasync def _():\n    pass\n\n# 当函数被调用时汇报到群聊\n@sender.when_func_call(log)\nasync def _():\n    pass\n\n```\n\n## 配置\n请在 `bot.py` 同一目录下创建文件 `secure.yaml` (注意后缀名) , 然后根据自己的情况填写配置 (目录下有一个示例)\n```yaml\nconfig:\n  basic:\n    reply_id: # !必须换成一个群号 , 是用来报告一些管理相关信息的群聊号码\n    bot_name: # bot 名\n    debug: false # 是否开启 `debug` 模式 (影响`sqla`的`echo`设置)\n    db_link: "sqlite+aiosqlite:///admin.sqlite" # sqla 数据库链接\n    ignore_global_control: # 忽略管控的插件名称\n      - nonebot_plugin_PCtrl\n\n  plugins:\n    # 配置越具体的优先级越高 , 同一具体的优先级 群 > qq号 > 插件名称\n    glob: # 使用 glob 代表所有插件\n      - space: 0 # 使用 0 代表私聊\n        coolen_time: 10 # 冷却时间\n      - space: 123456 # 否则代表群号为该数字的群\n        coolen_time: 10\n\n    echo: # 否则使用插件名(以 nonebot 加载显示的为准)\n      - space: 0\n        coolen_time: 10\n      - space: 1919810\n        coolen_time: 10\n\n```\n\n因为使用了特殊的语句~~(`mysql`泰拉)~~ , 所以以下方言二选一使用\n1. `postgresql+asyncpg` 需要安装 `asyncpg`\n2. `sqlite+aiosqlite` 默认依赖\n\n## 特别感谢\n\n[nonebot2](https://github.com/nonebot/nonebot2) : 优秀的跨平台 python 异步机器人框架\n\n[SQLAlchemy](https://www.sqlalchemy.org/) : 完善~~文档根本看不懂~~的 ORM\n\n## 部分更新\n\n- `0.2.0` : 适配 `b1` , ~~优化整体逻辑~~\n- `0.1.8` : 冷却时间信息显示与命令修改功能完成\n- `0.1.6` : 全局冷却功能完成',
    'author': 'CherryGS',
    'author_email': 'uplliode@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/CherryGS/nonebot_plugin_PCtrl',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
